using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Windows.Forms;

namespace BuscaPreco
{
    class Servidor
    {
        // Cria o evento para receber comandos
        public delegate void onReceiveCommand(object sender, string comando);
        public event onReceiveCommand onReceive;

        // Cria o evento para indicar alteraes na lista de terminais
        public delegate void onChangeList(ArrayList lista);
        public event onChangeList onChange;

        private Socket server;// Socket principal abre a porta 6500 para conexo dos terminais
        private Socket cliente;// Socket do cliente
        private ArrayList listaTerminais;// Lista com os terminais conectados
        private Thread threadServidor;//thread que  usada para espera dos terminais
        private IPEndPoint IPServer;// configurao do IP do servidor

        /*
         Mtodo: ReceiveCommand
         Funo: Trata os eventos dos terminais
         
         Entrada: sender - Indicando o terminal que enviou o evento
                  str - string com a mensagem do terminal
         */
        private void ReceiveCommand(object sender, string str) {
            if (onReceive != null) // verifica se h alguma funo para tratar o evento
                onReceive(sender, str); // dispara o evento
        }

        /*
         Mtodo: RemoveTerminal
         Funo: Remove um terminal (desconectado) da lista
         
         Entrada: sender - o terminal que ser removido
         */
        private void RemoveTerminal(object sender) {
            listaTerminais.Remove((Terminal)sender); // Remove o terminal
            if (onChange != null) // verifica se h alguma funo para o tratamento do evento
                onChange(listaTerminais); // dispara o evento
        }

        /*
         Mtodo: AddTerminal
         Funo: Adiciona um terminal na lista
         
         Entrada: term - O terminal a ser adicionado na lista
         */
        private void AddTerminal(Terminal term) {
            listaTerminais.Add(term);//adiciona o terminal
            if (onChange != null)// verifica se existe a funo para tratar o evento
                onChange(listaTerminais);// dispara o evento
        }

        /*
         Mtodo: Servidor
         Funo: Construtor da classe Servidor
         */
        public Servidor() {
            listaTerminais = new ArrayList(); // cria o objeto da lista
            server = new Socket(AddressFamily.InterNetwork,SocketType.Stream,ProtocolType.Tcp);//cria/configura o socket servidor
            IPServer = new IPEndPoint(IPAddress.Any, 6500);// configura a porta e o IP do servidor
        }

        /*
        Mtodo: startServer
        Funo: Funo que inicia o servidor
        */
        public void startServer()
        {
            threadServidor = new Thread(ProcessaServidor);// cria uma thread para esperar conexes informando qual funo ser executada
            threadServidor.Start();// inicia a thread
        }

        /*
         Mtodo: ProcessaServidor
         Funo: Funo executada em uma thread para esperar por conexes dos terminais
         */
        private void ProcessaServidor(){
            Terminal terminal; // cria uma instncia para a classe Terminal
            server.Bind(IPServer); // Configura a porta do servidor
            server.Listen(5); // abre a porta para conexes

            // loop infinito que espera as conexes dos terminais
            while (true){
                cliente = server.Accept();// aceita a conexo do terminal e retorna o socket para comunicao
                if (cliente.Connected) // se houve a correta conexo do terminal
                {
                    terminal = new Terminal(cliente);// cria o objeto terminal e passa o socket para o objeto
                    terminal.onReceive += new Terminal.onReceiveCommand(ReceiveCommand);//configura a funo para a qual o terminal ir disparar o evento de comandos
                    terminal.Desconectar += new Terminal.onDisconectTerminal(RemoveTerminal);//configura a funo para a qual o terminal ir disparar o evento de desconexo
                    Thread.Sleep(2000); // espera 2 segundos para o terminal comear a comuniocao
                    AddTerminal(terminal);// adiciona o terminal  lista
                }
            }
        }
    }
}
